@@ -859,11 +859,10 @@ static void zcode_op_output_stream(ZStack* stack,
       if (machine.transcript_file == NULL)
 	{
 	  machine.transcript_file = get_file_write(NULL, script_fname, ZFile_transcript);
+	  write_stringf(machine.transcript_file, "*** Transcript generated by Zoom\n\n");	
 	}
 
       if (machine.transcript_file != NULL) {
-	write_stringf(machine.transcript_file, "*** Transcript generated by Zoom\n\n");
-
 	machine.transcript_on = 1;
 
 	w = Word(ZH_flags2);
@@ -1023,9 +1022,25 @@ static void zcode_op_aread_5678(ZDWord* pc,
   ZByte* mem;
   unsigned int* buf;
   int x;
+  int bufLen;
   
   mem = machine.memory + (ZUWord) args->arg[0];
-  buf = malloc(sizeof(int)*(mem[0]+1));
+
+  if (((ZUWord)args->arg[0]) < 64) {
+    zmachine_warning("zcode_op_aread called with a buffer in the header area!");
+  }
+  
+  if (mem[0] <= 0) {
+    zmachine_fatal("zcode_op_aread called with the memory buffer size set to %i", mem[0]);
+  }
+  
+  if (mem[1] > mem[0]) {
+    zmachine_warning("zcode_op_aread called with an invalid buffer: the number of characters the buffer already contains must be less than the total length of the buffer");
+    return;
+  }
+  
+  bufLen = mem[0];
+  buf = malloc(sizeof(int)*(bufLen+1));
 
   if (args->arg[7] != 0)
     {
@@ -1039,7 +1054,6 @@ static void zcode_op_aread_5678(ZDWord* pc,
 	{
 	  mem[1] = 0;
 	  free(buf);
-	  return;
 	}
     }
   
@@ -1048,7 +1062,7 @@ static void zcode_op_aread_5678(ZDWord* pc,
       /* zmachine_warning("aread: using existing buffer (display may
        * get messed up)"); */
       
-      for (x=0; x<mem[1]; x++)
+      for (x=0; x<mem[1] && x<mem[0]; x++)
 	{
 	  buf[x] = zscii_unicode[mem[x+2]];
 	}
@@ -1091,7 +1105,7 @@ static void zcode_op_aread_5678(ZDWord* pc,
     {
       int res;
       
-      res = stream_readline((int*)buf, mem[0], 0);
+      res = stream_readline((int*)buf, bufLen, 0);
       display_terminating(NULL);
 
       if (res == 254 || res == 253)
@@ -1124,7 +1138,7 @@ static void zcode_op_aread_5678(ZDWord* pc,
     {
       int res;
 
-      res = stream_readline((int*)buf, mem[0], args->arg[2]*100);
+      res = stream_readline((int*)buf, bufLen, args->arg[2]*100);
       display_terminating(NULL);
       store(stack, st, res);
       
