@@ -287,10 +287,14 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	skein = [sk retain];
 	[layout setRootItem: [sk rootItem]];
 	
-	[[NSNotificationCenter defaultCenter] addObserver: self
-											 selector: @selector(skeinDidChange:)
-												 name: ZoomSkeinChangedNotification
-											   object: skein];
+	if (skein) {
+		// Fixed by Collin Pieper: adding an observer for nil has somewhat unwanted side effects
+		[[NSNotificationCenter defaultCenter] addObserver: self
+												 selector: @selector(skeinDidChange:)
+													 name: ZoomSkeinChangedNotification
+												   object: skein];
+	}
+	
 	[self setSelectedItem: nil];
 	[self skeinNeedsLayout];
 	
@@ -782,8 +786,12 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	ZoomSkeinItem* parent = [skein activeItem];
 	while (parent != nil) {
 		if (parent == skeinItem) {
-			// Can't delete an item that's the parent of the active item
-			NSBeep(); // Maybe need some better feedback
+			if (![delegate respondsToSelector: @selector(cantDeleteActiveBranch)]) {
+				// Can't delete an item that's the parent of the active item
+				NSBeep();
+			} else {
+				[delegate cantDeleteActiveBranch];
+			}
 			return;
 		}
 		
@@ -938,7 +946,12 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	
 	if ([skeinItem parent] == nil) {
 		// Can't edit the root item
-		NSBeep();
+		if (![delegate respondsToSelector: @selector(cantEditRootItem)]) {
+			NSBeep();
+		} else {
+			[delegate cantEditRootItem];
+		}
+
 		return;
 	}
 	
@@ -1006,7 +1019,7 @@ NSString* ZoomSkeinItemPboardType = @"ZoomSkeinItemPboardType";
 	[fieldEditor setFont: [itemTextAttributes objectForKey: NSFontAttributeName]];
 	
 	[fieldEditor setRichText:NO];
-	[fieldEditor setAllowsDocumentBackgroundColorChange:NO];
+	if ([fieldEditor respondsToSelector: @selector(setAllowsDocumentBackgroundColorChange:)]) [fieldEditor setAllowsDocumentBackgroundColorChange:NO];
 	[fieldEditor setBackgroundColor:[NSColor whiteColor]];
 	
 	[[fieldEditor textContainer] setContainerSize: NSMakeSize(NSInsetRect(itemFrame, 2.0, 2.0).size.width, 1e6)];
