@@ -17,6 +17,10 @@
 #import "ZoomScrollView.h"
 #import "ZoomConnector.h"
 
+// Sets variables to force extreme memory checking in the Zoom task; this provides a fairly huge performance
+// decrease, but provides 'earliest possible' warning of heap corruption.
+#undef ZoomTaskMaximumMemoryDebug
+
 @implementation ZoomView
 
 static ZoomView** allocatedViews = nil;
@@ -58,6 +62,7 @@ static void finalizeViews(void) {
 		
 		// Input source
 		inputSource = nil;
+		inputPos = 0;
 
         // No upper/lower windows
         upperWindows = [[NSMutableArray allocWithZone: [self zone]] init];
@@ -91,7 +96,8 @@ static void finalizeViews(void) {
 
         [textScroller setHasHorizontalScroller: NO];
         [textScroller setHasVerticalScroller: YES];
-		[textScroller setBackgroundColor: [NSColor clearColor]];
+		[textScroller setBackgroundColor: [NSColor whiteColor]];
+		[textScroller setDrawsBackground: YES];
         
         NSSize contentSize = [textScroller contentSize];
 
@@ -414,6 +420,7 @@ static void finalizeViews(void) {
 		if (len > ((int)preserve + 2048)) {
 			// Need to truncate
 			[[textView textStorage] deleteCharactersInRange: NSMakeRange(0, len - preserve)];
+			inputPos -= len-preserve;
 		}
 	}
 	
@@ -534,6 +541,8 @@ static void finalizeViews(void) {
 			[textView setEditable: YES];
 
 			[self resetMorePrompt];
+		} else {
+			[textView setEditable: NO];
 		}
 		
 		// If we're using the upper window, run using the inputLine system
@@ -544,7 +553,8 @@ static void finalizeViews(void) {
 		// Scroll, input
 		if (!isUpperWindow) {
 			[self scrollToEnd];
-			inputPos = [[textView textStorage] length];
+			// inputPos = [[textView textStorage] length];
+			[self textStorageDidProcessEditing: nil];
 		}
 	} else {
 		// == Version 6 pixmap entry routines ==
@@ -629,7 +639,6 @@ static void finalizeViews(void) {
 - (void) stopReceiving {
     receiving = NO;
     receivingCharacters = NO;
-    [textView setEditable: NO];
 	[[textScroller upperWindowView] setFlashCursor: NO]; 
 	[self resetMorePrompt];
 }
@@ -683,6 +692,22 @@ static void finalizeViews(void) {
 
 // = Utility functions =
 
+- (void) writeAttributedString: (NSAttributedString*) string {
+	// Writes the given string to the lower window
+	[[textView textStorage] insertAttributedString: string
+										   atIndex: inputPos];
+	inputPos += [string length];
+}
+
+- (void) clearLowerWindowWithStyle: (ZStyle*) style {
+    [[[textView textStorage] mutableString] replaceCharactersInRange: NSMakeRange(0, inputPos)
+																	 withString: @""];
+    [textView setBackgroundColor: [style reversed]?[self foregroundColourForStyle: style]:[self backgroundColourForStyle: style]];
+    [textView clearPastedLines]; 
+	
+	inputPos = 0;
+}
+
 - (void) scrollToEnd {
 	if (editingTextView) {
 		// Re-queue for later
@@ -1364,7 +1389,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         sepHeight -= [upperWindowBuffer containerSize].height;
 		
         if ([[textView textStorage] length] == 0) {
-            [[[textView textStorage] mutableString] appendString: @"\n"];
+            [[[textView textStorage] mutableString] insertString: @"\n"
+														 atIndex: inputPos];
+			inputPos++;
         }
 
         do {
@@ -1379,7 +1406,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
                                                                  inTextContainer: [textView textContainer]];
 
             if (NSMinY(endRect) < sepHeight) {
-                [[textView textStorage] appendAttributedString: newLine];
+                [[textView textStorage] insertAttributedString: newLine
+													   atIndex: inputPos];
+				inputPos += [newLine length];
             } else {
                 break;
             }
@@ -1434,7 +1463,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     if (upperWindowBuffer == nil) return;
 
     if ([[textView textStorage] length] == 0) {
-        [[[textView textStorage] mutableString] appendString: @"\n"];
+        [[[textView textStorage] mutableString] insertString: @"\n"
+													 atIndex: inputPos];
+		inputPos++;
     }
     
     do {
@@ -1450,7 +1481,9 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
         theContainer = [[textView layoutManager] textContainerForGlyphAtIndex: endGlyph.location effectiveRange: &eRange];
 
         if (theContainer == upperWindowBuffer) {
-            [[[textView textStorage] mutableString] appendString: @"\n"];
+            [[[textView textStorage] mutableString] insertString: @"\n"
+														 atIndex: inputPos];
+			inputPos++;
         }
 
         // I suppose there's an outside chance of an infinite loop here
@@ -1512,6 +1545,23 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 	// Start a new machine
     zoomTask = [[NSTask allocWithZone: [self zone]] init];
     zoomTaskData = [[NSMutableString allocWithZone: [self zone]] init];
+	
+#ifdef ZoomTaskMaximumMemoryDebug
+	NSMutableDictionary* taskEnvironment = [[[NSProcessInfo processInfo] environment] mutableCopy];
+	
+	[taskEnvironment setObject: @"1"
+						forKey: @"MallocCheckHeapStart"];
+	[taskEnvironment setObject: @"1"
+						forKey: @"MallocCheckHeapEach"];
+	[taskEnvironment setObject: @"1"
+						forKey: @"MallocScribble"];
+	[taskEnvironment setObject: @"1"
+						forKey: @"MallocGuardEdges"];
+	[taskEnvironment setObject: @"YES"
+						forKey: @"NSZombieEnabled"];
+	
+	[zoomTask setEnvironment: [taskEnvironment autorelease]];
+#endif
 
     if (serverName == nil) {
         serverName = [[NSBundle mainBundle] pathForResource: @"ZoomServer"
@@ -1595,6 +1645,8 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
     [[textView textStorage] appendAttributedString: newline];
     [[textView textStorage] appendAttributedString: string];
     [[textView textStorage] appendAttributedString: newline];
+	inputPos = [[textView textStorage] length];
+	[textView setEditable: NO];
 
     // Update the windows
     [self rearrangeUpperWindows];
@@ -1731,7 +1783,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 			}
             typeCode = 'IFZS';
             if (supportsMessage) {
-                [panel setMessage: [NSString stringWithFormat: @"%@ savegame (quetzal) file", saveOpen]];
+                [panel setMessage: [NSString stringWithFormat: @"%@ saved game (quetzal) file", saveOpen]];
                 [panel setAllowedFileTypes: [NSArray arrayWithObjects: usePackage?@"zoomSave":@"qut", nil]];
             }
             break;
@@ -2205,6 +2257,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		
 		// Workaround for a Cocoa bug
 		[[textView textStorage] setAttributedString: [[[NSAttributedString alloc] initWithAttributedString: storage] autorelease]];
+		inputPos = [[textView textStorage] length];
 		
 		// Final setup
 		upperWindowsToRestore = [upperWindows count];
@@ -2242,6 +2295,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 		
 		// Workaround for a Cocoa bug
 		[[textView textStorage] setAttributedString: [[[NSAttributedString alloc] initWithAttributedString: storage] autorelease]];
+		inputPos = [[textView textStorage] length];
 		
 		commandHistory = [[decoder decodeObject] retain];
 		if (autosaveVersion == 101) pixmapWindow = [[decoder decodeObject] retain];
@@ -2316,6 +2370,7 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 			[[textView textStorage] beginEditing];
 			// Workaround for a bug in Cocoa
 			[[textView textStorage] setAttributedString: [[[NSAttributedString alloc] initWithAttributedString: storage] autorelease]];
+			inputPos = [[textView textStorage] length];
 			
 			commandHistory = [[decoder decodeObject] retain];
 			
@@ -2499,6 +2554,57 @@ shouldChangeTextInRange:(NSRange)affectedCharRange
 	}
 }
 
+- (NSString*) receivedTextToDate {
+	if (inputLine != nil) {
+		return [inputLine inputLine];
+	} else {
+		NSString* str = [[textView textStorage] string];
+		
+		return [str substringWithRange: NSMakeRange(inputPos,
+													[str length]-inputPos)];
+	}
+}
+
+- (NSString*) backtrackInputOver: (NSString*) prefix {
+	if (prefix == nil) return nil;
+	
+	if (inputLine != nil) {
+		// Input lines currently are unable to backtrack, so the prefix remains unaltered
+		return prefix;
+	} else {
+		NSString* str = [[textView textStorage] string];
+		
+		// 'len' contains the amount of text we'll attempt to backtrack across
+		int len = [prefix length];
+		if (len > [str length]) len = [str length];
+		
+		// Cut out a substring according to the length
+		str = [str substringWithRange: NSMakeRange([str length]-len, len)];
+		
+		// We compare lowercase versions of the string: this allows for things like Beyond Zork which write 'EXAMINE' but add 'examine' to the buffer
+		NSString* lowerPrefix = [prefix lowercaseString];
+		str = [str lowercaseString];
+		
+		// See how far back we can go
+		int offset = len-1;
+		int prefixOffset = [lowerPrefix length]-1;
+		
+		while (offset >= 0 && prefixOffset >= 0 && [lowerPrefix characterAtIndex: prefixOffset] == [str characterAtIndex: offset]) {
+			offset--;
+			prefixOffset--;
+		}
+		
+		// offset, prefixOffset indicate the last character that wasn't matched
+		offset++;
+		
+		int matchLength = len-offset;
+		
+		inputPos -= matchLength;
+		
+		return [prefix substringToIndex: offset];
+	}
+}
+
 // = Output receivers =
 
 - (void) addOutputReceiver: (id) receiver {
