@@ -88,7 +88,7 @@
 }
 
 - (NSString*) description {
-    return @"Zoom 1.0.3 ZMachine object";
+    return @"Zoom 1.0.4 ZMachine object";
 }
 
 - (void) connectionDied: (NSNotification*) notification {
@@ -193,6 +193,8 @@
     // Setup the display, etc
     rc_set_game(zmachine_get_serial(), Word(ZH_release), Word(ZH_checksum));
     display_initialise();
+	
+	if (wasRestored) zmachine_setup_header();
 
     // Start running the machine
     switch (machine.header[0])
@@ -771,7 +773,7 @@ static NSString* zscii_to_string(ZByte* buf) {
 @end
 
 // = Fatal errors and warnings =
-void  zmachine_fatal(char* format, ...) {
+void zmachine_fatal(char* format, ...) {
 	char fatalBuf[512];
 	va_list  ap;
 	
@@ -781,12 +783,12 @@ void  zmachine_fatal(char* format, ...) {
 	
 	fatalBuf[511] = 0;
 	
-	[[mainMachine display] displayFatalError: [NSString stringWithCString: fatalBuf]];
+	[[mainMachine display] displayFatalError: [NSString stringWithFormat: @"%s (PC=#%x)", fatalBuf, machine.zpc]];
 	
 	display_exit(1);
 }
 
-void  zmachine_warning(char* format, ...) {
+void zmachine_warning(char* format, ...) {
 	char fatalBuf[512];
 	va_list  ap;
 	
@@ -800,5 +802,5 @@ void  zmachine_warning(char* format, ...) {
 	NSLog(@"Warning: %s", fatalBuf);
 #endif
 	
-	[[mainMachine display] displayWarning: [NSString stringWithCString: fatalBuf]];
+	[[mainMachine display] displayWarning: [NSString stringWithFormat: @"%s (PC=#%x)", fatalBuf, machine.zpc]];
 }
