@@ -6,7 +6,7 @@
 //  Copyright (c) 2003 Andrew Hunter. All rights reserved.
 //
 
-#define DEBUG_BUILD
+#undef  DEBUG_BUILD
 
 #import <Cocoa/Cocoa.h>
 
